////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit IEDResManager;

interface
uses Windows, Messages, SysUtils, Classes,
     IdGlobal, IdTCPConnection, IdTCPClient,
     DAVresources, IEDXML;

type
   TOnErrorEvent = procedure(Sender: TComponent; errno: word; Errmsg: string) of object;

   TLockType     = (lockRead, lockWrite, lockEmpty);
   TLockScope    = (lockShared,lockExclusive);
   TLockScopeSet = set of TLockScope;

Const
   DAVLockTypeStr  : array[TLockType] of string = ('read','write','');
   DAVLockScopeStr : array[TLockScope] of string = ('shared','exclusive');

Type
   TSupportedLock = class
   public
    supportWrite  : TLockScopeSet; // ES
    supportRead   : TLockScopeSet; // ES
    constructor Create;
    procedure   ParseXMLnode(Node:TIEDXMLNode);
    function    IsSupportLock(LockType:TLockType; LockScope:TLockScope):boolean;
   end;

   TLockEntry = class
   public
    lockType    : TLockType;
    lockScope   : TLockScope;
    lockOwner   : string;
    lockToken   : string;
    lockDepth   : string;
    lockTimeout : string;
    constructor Create;
    procedure   ParseXMLnode(Node:TIEDXMLNode);
   end;

   TResource = class
    public
     rspHREF: string;
     rspDisplayName: string;
     rspSize: integer;
     rspCreationDate: TDateTime;
     rspLastModified: TDateTime;
     rspContentLength: integer;
     rspContentType: string;
     rspETag : string;
     rspIsHidden: integer;
     rspIsCollection : integer;
     rspStatus : string;
     //Additional items
     Index         : integer;
     ResourceName  : string;
     DisplayName   : string;
     SupportedLock : TSupportedLock;
     LockEntry     : TLockEntry;

     constructor Create;
     destructor  Destroy; override;
     procedure   ParseXMLnode(Node:TIEDXMLNode);
     procedure   CheckData(host,path:string);
     function    IsCollection: boolean;
     function    StatusCode: integer;
     function    StatusStr: string;
     function    IsOK: boolean;
    end;

   TCustomResourceManager = class(TPersistent)
   private
    FCurrentDir: string;
    FOnError: TOnErrorEvent;
    FOnChangeDir: TNotifyEvent;
    FResources : TStrings;
    FSelectedIndex: integer;

    function  GetCurrentDir: string; virtual;
    procedure SetCurrentDir(const Value: string); virtual;
    function  GetResources(Index: integer): TResource;  virtual;
    procedure SetSelectedIndex(const Value: integer);  virtual;
    function  GetSelectedResource: TResource;
    function  GetResourcesCount: integer;
   protected
    function AddNewResource(HREF:string):TResource;
   public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure LoadResourceList; virtual;
    function  MkDir(NewName: string): boolean; virtual; abstract;
    function  Delete: boolean; virtual; abstract;
    function  Copy(NewName: string): boolean; virtual; abstract;
    function  Move(NewName: string): boolean; virtual; abstract;
    function  Download(LocalName: string): boolean; virtual; abstract;
    function  Upload(LocalName: string): boolean; virtual; abstract;
    function  Lock( LockOwner: string;
                    LockType: TLockType = lockWrite;
                    LockScope: TLockScope= lockExclusive): boolean; virtual; abstract;
    function  Unlock: boolean; virtual; abstract;

    property CurrentDir: string read GetCurrentDir write SetCurrentDir;
    property ResourceList: TStrings read FResources;
    property Resources[Index:integer]:TResource read GetResources; default;
    property ResourcesCount:integer read GetResourcesCount;
    property SelectedIndex:integer read FSelectedIndex write SetSelectedIndex;
    property SelectedResource:TResource read GetSelectedResource;
    property OnError: TOnErrorEvent read FOnError write FOnError;
    property OnChangeDir: TNotifyEvent read FOnChangeDir write FOnChangeDir;
   end;

   TRemoteResourceManager = class(TCustomResourceManager)
   private
    FUser: string;
    FPwd: string;
    FPort: integer;
    FHost: string;
    FContent: TStream;
    FCHeader: TStrings;
    FRHeader: TStrings;
    FResponse: TStream;
    FResponseStatusStr: string;
    FResponseCode: integer;
    FResponseStr: string;

    FContentType:string;

    procedure SetHost(const Value: string); virtual;
    procedure SetPassword(const Value: string); virtual;
    procedure SetPort(const Value: integer); virtual;
    procedure SetUser(const Value: string); virtual;
   protected
    procedure CreateRequest(cmd : integer; ResourceName : string =''); virtual;
    procedure SendRequest(AContent,AResponse:TStream);  virtual;  abstract;
   public
    constructor Create; override;
    destructor  Destroy; override;
    function    ConnectEx(Host,Port,User,Pwd: string): boolean; virtual;
    function    Connect: boolean; virtual; abstract;
    function    Disconnect: boolean; virtual; abstract;

    property CHeader  : TStrings read FCHeader;
    property Content  : TStream read FContent;
    property RHeader  : TStrings read FRHeader;
    property Response : TStream read FResponse;
    property ResponseCode : integer read FResponseCode;
    property ResponseStr  : string read FResponseStr;
    property Host: string read FHost write SetHost;
    property Port: integer read FPort write SetPort;
    property User: string read FUser write SetUser;
    property Password: string read FPwd write SetPassword;
   end;

  TIEDWebDav = class(TRemoteResourceManager)
  private
    FConnection: TIdTCPClient;
    FXML        : TIEDXMLReader;
    FChunked    : boolean;
    FContentLen : integer;
    procedure ReadResponseCode;
    procedure ReadHeaders;
    procedure CheckIsSelected;
  protected
    procedure CreateRequest(cmd : integer; ResourceName : string =''); override;
    procedure SendRequest(AContent,AResponse:TStream); override;
  public
    function  Connect: boolean;  override;
    function  Disconnect: boolean;  override;
    procedure LoadResourceList; override;
    function  MkDir(NewName: string): boolean; override;
    function  Delete: boolean; override;
    function  Copy(NewName: string): boolean; override;
    function  Move(NewName: string): boolean; override;
    function  Download(LocalName: string): boolean; override;
    function  Upload(LocalName: string): boolean; override;
    function  Lock( LockOwner: string;
                    LockType: TLockType = lockWrite;
                    LockScope: TLockScope= lockExclusive): boolean; override;
    function  Unlock: boolean; override;
  public
    property Connection: TIdTCPClient read FConnection write FConnection;
    property XMLreader : TIEDXMLReader read FXML;

    constructor Create; override;
    destructor  Destroy; override;
  end;

function UpDirectory(Const Dir:string):string;

implementation

function UpDirectory(Const Dir:string):string;
var
   r,l: integer;
begin
 l:= Length(Dir); r:=0;
 if l>0 then
 begin
  if Dir[l]<>'/' then
   r:= RPos('/',Dir)
  else
   r:= RPos('/',Dir,l-1);
 end;
 if r<>0 then
  result:=system.Copy(Dir,1,r)
 else
  result:='/';
end;

function UNCToDateTime(UNC:string):TDateTime;
begin //2002-08-12T14:24:16
 result:=0;
 try
   result:= encodedate(strtoint(copy(UNC,1,4)),strtoint(copy(UNC,6,2)),strtoint(copy(UNC,9,2)))+
            encodetime(strtoint(copy(UNC,12,2)),strtoint(copy(UNC,15,2)),strtoint(copy(UNC,18,2)),0)+
            OffSetFromUTC;
 except
 end;
end;

{ TSupportedLock }
constructor TSupportedLock.Create;
begin //default support all locks
  supportWrite:=[lockShared,lockExclusive];
  supportRead :=[lockShared,lockExclusive];
end;

procedure TSupportedLock.ParseXMLnode(Node: TIEDXMLNode);
var
   i , j: integer;
   R,W,S,E : boolean;
begin
 supportWrite:=[];
 supportRead :=[];
 R:=false; W:=false; S:=false; E:=false;
 for i:=0 to Node.Nodes.Count-1 do
  with Node.Nodes[i] do
   if IsSameName(':LOCKENTRY') then
    begin
     With Node.Nodes[i] do
      for j:=0 to Nodes.Count -1 do
       begin
        if Nodes[j].IsSameName(':WRITE') then W:=true;
        if Nodes[j].IsSameName(':READ') then R:=true;
        if Nodes[j].IsSameName(':SHARED') then S:=true;
        if Nodes[j].IsSameName(':EXCLUSIVE') then E:=true;
       end;
       if W then
       begin
        if S then supportWrite:=supportWrite + [lockShared];
        if E then supportWrite:=supportWrite + [lockExclusive];
       end;
       if R then
       begin
        if S then supportRead:=supportWrite + [lockShared];
        if E then supportRead:=supportWrite + [lockExclusive];
       end;
    end;
end;

function TSupportedLock.IsSupportLock(LockType:TLockType;
                                      LockScope:TLockScope): boolean;
begin
  if LockType = lockRead then
   result:=(LockScope in supportRead)
  else
   result:=(LockScope in supportWrite);
end;

{ TLockEntry }

constructor TLockEntry.Create;
begin
  lockType    := lockEmpty;
  lockScope   := lockShared;
  lockOwner   := '';
  lockToken   := '';
  lockDepth   := '';
  lockTimeout := '';
end;

procedure TLockEntry.ParseXMLnode(Node: TIEDXMLNode);
var
   RNode, TNode : TIEDXMLNode;
   i, j : integer;
begin
 RNode:= Node.NodeOf(':ACTIVELOCK',[schPartial,schUpper],8);
 if RNode=nil then exit;
 for i:=0 to RNode.Nodes.Count-1 do
  with RNode.Nodes[i] do
   begin
    if IsSameName(':LOCKTYPE') then
     begin
       if NodeOf(':WRITE',[schPartial,schUpper,schNoSelf],2)<>nil then
        lockType := lockWrite
       else
       if NodeOf(':READ',[schPartial,schUpper,schNoSelf],2)<>nil then
        lockType := lockRead;
     end;
    if IsSameName(':LOCKSCOPE') then
     begin
       if NodeOf(':SHARE',[schPartial,schUpper,schNoSelf],2)<>nil then
        lockScope := lockShared
       else
       if NodeOf(':EXCLUSIVE',[schPartial,schUpper,schNoSelf],2)<>nil then
        lockScope := lockExclusive;
     end;
    if IsSameName(':LOCKTOKEN') then
     begin
       TNode:= NodeOf(':HREF',[schPartial,schUpper,schNoSelf],2);
       if TNode<>nil then
        lockToken :=  TNode.Value;
     end;
    if IsSameName(':OWNER') then
     begin
      lockOwner := 'owner='+Value+#13;
       for j:=0 to Nodes.Count-1 do
        lockOwner := lockOwner + Nodes[j].Name +'='+Nodes[j].Value+#13;
     end;
    if IsSameName(':DEPTH') then
     lockDepth :=  Value;
    if IsSameName(':TIMEOUT') then
     lockTimeout :=  Value;
   end;
end;

{ TResource }

constructor TResource.Create;
begin
 inherited;
     rspHREF:='';
     rspDisplayName:='';
     rspSize:=-1;
     rspCreationDate:=-1;
     rspLastModified:=-1;
     rspContentLength:=-1;
     rspContentType:='';
     rspETag:='';
     rspStatus:='';
     rspIsHidden:=-1;
     rspIsCollection:=-1;
     SupportedLock := TSupportedLock.Create;
     LockEntry     := TLockEntry.Create;
end;

destructor TResource.Destroy;
begin
  if Assigned(SupportedLock) then SupportedLock.Free;
  if Assigned(LockEntry) then LockEntry.Free;
  inherited;
end;

procedure TResource.ParseXMLnode(Node: TIEDXMLNode);
var
   RNode : TIEDXMLNode;
   i : integer;
begin
   RNode:= Node.NodeOf(':STATUS',[schPartial,schUpper],8);
   if RNode<>nil then rspStatus:= RNode.Value;
   if IsOK then
   begin
     RNode:= Node.NodeOf(':PROPSTAT',[schPartial,schUpper],8);
     if RNode<>nil then
     begin
      RNode:= RNode.NodeOf(':PROP',[schPartial,schUpper,schNoSelf],4);
        for i:=0 to RNode.Nodes.Count-1 do
         with RNode.Nodes[i] do
          begin
            if IsSameName(':GETCONTENTLEN') then
              rspContentLength := StrToInt(Value);
            if IsSameName(':DISPLAYNAME') then
              rspDisplayName := Value;
            if IsSameName(':GETETAG') then
              rspETag := Value;
            if IsSameName(':ISHIDDEN') then
              rspIsHidden := StrToInt(Value);
            if IsSameName(':ISCOLLECTION') then
              rspIsCollection := StrToInt(Value);
            if IsSameName(':GETCONTENTTYPE') then
              rspContentType := Value;
            if IsSameName(':GETLASTMODIF') then
              rspLastModified := GMTToLocalDateTime(Value);
            if IsSameName(':CREATIONDATE') then
              rspCreationDate := UNCToDateTime(Value);
            if IsSameName(':SUPPORTEDLOCK') then
              SupportedLock.ParseXMLnode(RNode.Nodes[i]);
            if IsSameName(':LOCKDISCOVERY') then
              LockEntry.ParseXMLnode(RNode.Nodes[i]);
            if IsSameName(':RESOURCETYPE') then
             if RNode.Nodes[i].NodeOf(':COLLECTION',[schPartial,schUpper],2)<>nil then
              rspIsCollection:=1;
           end;
     end;
   end;
end;

procedure TResource.CheckData;
var
  l : integer;
begin
   l:=Pos(host,rspHREF);
    if l=0 then
      l:=1
    else
      l:= l+length(host);

   ResourceName :=system.copy(rspHREF,l,length(rspHREF)-l+1);

   if Length(Path)>1 then
    begin
       l:= pos(path,ResourceName);
     if l=0 then
       l:=1
     else
       l:= l+Length(path);
    end
   else
    if ResourceName[1]='/' then
      l:=2
    else
      l:=1;

  DisplayName :=UrlDecode(system.copy(ResourceName,l,length(ResourceName)-l+1));
  l:= length(DisplayName);
  if (l>0)and(DisplayName[l]='/') then SetLength(DisplayName,l-1);
  if DisplayName='' then DisplayName:='.';
end;


function TResource.IsCollection: boolean;
begin
 if rspIsCollection<0 then
   result:= (Pos('-DIRECTORY',UpperCase(rspContentType))<>0)
 else
   result:= (rspIsCollection<>0);
end;

function TResource.StatusCode: integer;
begin
 if (system.Copy(rspStatus, 1, 5)='HTTP/') then
   result:= StrToInt(Trim(system.Copy(rspStatus,10,4)))
 else
   result:= -1;
end;

function TResource.StatusStr: string;
begin
 if (system.Copy(rspStatus, 1, 5)='HTTP/') then
   result:= Trim(system.Copy(rspStatus,14,Length(rspStatus)-13))
 else
   result:= '';
end;

function TResource.IsOK: boolean;
begin
 result:= (StatusCode = 200);
end;

{ TCustomResourceManager }

constructor TCustomResourceManager.Create;
begin
  inherited;
  FCurrentDir := '/';
  FResources := TStringList.Create;
end;

destructor TCustomResourceManager.Destroy;
begin
  if Assigned(FResources) then FResources.Free;
  inherited;
end;

function TCustomResourceManager.AddNewResource(HREF: string): TResource;
begin
 result:= TResource.Create;
 result.rspHREF := HREF;
 result.Index := FResources.AddObject(HREF,result);
end;

function TCustomResourceManager.GetResources(Index: integer): TResource;
begin
 If (Index>FResources.Count-1)or(Index<0) then
  result:=nil
 else
  result:= TResource(FResources.Objects[Index]);
end;

function TCustomResourceManager.GetCurrentDir: string;
begin
 Result:=FCurrentDir;
end;

procedure TCustomResourceManager.SetCurrentDir(const Value: string);
begin
 if FCurrentDir<> value then
  begin
   FCurrentDir:= value;
   if Assigned(FOnChangeDir) then
    FOnChangeDir(Self);
  end;
end;

procedure TCustomResourceManager.SetSelectedIndex(const Value: integer);
begin
 if (FSelectedIndex <> Value)and
    (FSelectedIndex<FResources.Count)and
    (FSelectedIndex>=0) then
  FSelectedIndex := Value;
end;

function TCustomResourceManager.GetSelectedResource: TResource;
begin
 result:= Resources[FSelectedIndex];
end;

procedure TCustomResourceManager.LoadResourceList;
begin
 Fresources.Clear;
 FSelectedIndex:=0;
end;

function TCustomResourceManager.GetResourcesCount: integer;
begin
 result:= FResources.Count;
end;

{ TIEDWebDav }

constructor TRemoteResourceManager.Create;
begin
  inherited;
  FCHeader    := TStringList.Create;
  FRHeader    := TStringList.Create;
  FContent    := TMemoryStream.Create;
  FResponse   := TMemoryStream.Create;
  FContentType:='text/xml; charset="utf-8"';
end;

destructor TRemoteResourceManager.Destroy;
begin
  If Assigned(FCHeader) then FCHeader.Free;
  If Assigned(FRHeader) then FRHeader.Free;
  If Assigned(FContent) then FContent.Free;
  If Assigned(FResponse) then FResponse.Free;
  inherited;
end;

function TRemoteResourceManager.ConnectEx(Host, Port, User, Pwd: string): boolean;
begin
  FHost := Host;
  FPort := StrToInt(Port);
  FUser := User;
  FPwd  := Pwd;
  result:=Connect;
end;

procedure TRemoteResourceManager.CreateRequest;
begin
  FCHeader.Clear;
  FRHeader.Clear;
  FContent.Size :=0;
  FResponse.Size :=0;
end;

procedure TRemoteResourceManager.SetHost(const Value: string);
begin
  FHost := Value;
  CurrentDir := '/';
end;

procedure TRemoteResourceManager.SetPassword(const Value: string);
begin
  FPwd := Value;
end;

procedure TRemoteResourceManager.SetPort(const Value: integer);
begin
  FPort := Value;
end;

procedure TRemoteResourceManager.SetUser(const Value: string);
begin
  FUser := Value;
end;

{ TIEDWebDav }

constructor TIEDWebDav.Create;
begin
  inherited;
  FConnection:= TIdTCPClient.Create(nil);
  FXML  := TIEDXMLReader.Create;
  FHost := 'localhost';
  FPort := 80;
end;

destructor TIEDWebDav.Destroy;
begin
  if Assigned(FConnection) then FConnection.Free;
  If Assigned(FXML) then FXML.Free;
  inherited;
end;

function TIEDWebDav.Connect: boolean;
begin
 if FConnection.Connected then
  FConnection.Disconnect;
 FConnection.Host := FHost;
 FConnection.Port := FPort;
 FConnection.Connect;
 result:= FConnection.Connected;
end;

procedure TIEDWebDav.CreateRequest(cmd: integer; ResourceName : string ='');
begin
  inherited;
  FCHeader.Add(Format(davStrHeader,[davCmdStr[TDAVcmd(cmd)],ResourceName]));
  FCHeader.Add(Format(davStrHost,[FHost]));
  if FUser <> '' then
   FCHeader.Add(Format(davStrAuthor,[EncodeString(encBase64, FUser+':'+FPwd+#13#10)]));
end;

procedure TIEDWebDav.ReadHeaders;
var s,us : string;
begin
  FContentLen := 0;
  FChunked := false;
  s := FConnection.ReadLn;
  while Length(s) > 0 do
  begin
    FRHeader.Add(s);
    us := UpperCase(s);
    if (AnsiPos('CONTENT-LENGTH',us)<>0) then
      FContentLen:= StrToInt(trim(system.Copy(s,17,length(s)-16)));
    if ((AnsiPos('TRANSFER-ENCODING',us)<>0)and(AnsiPos('CHUNKED',us)<>0)) then
      FChunked := true;
    s := FConnection.ReadLn;
  end;
end;

procedure TIEDWebDav.ReadResponseCode;
begin
 FRHeader.Clear;
 FResponseStatusStr:= FConnection.ReadLn;
 if (system.Copy(FResponseStatusStr, 1, 5)='HTTP/') then
  begin
   FResponseCode:= StrToInt(Trim(system.Copy(FResponseStatusStr,10,4)));
   FResponseStr:= Trim(system.Copy(FResponseStatusStr,14,Length(FResponseStatusStr)-13));
  end
 else
  begin
   FResponseCode:= -1;
   FResponseStr:='';
  end;
 FRHeader.Add(FResponseStatusStr);
end;

procedure TIEDWebDav.CheckIsSelected;
begin
  if SelectedResource=nil then Exception.Create('Select resource first');
end;

procedure TIEDWebDav.SendRequest(AContent,AResponse:TStream);

  function BytesForRead: integer;
  var
    j: Integer;
    ss: string;
  begin
   ss := '$'+FConnection.ReadLn;
   j  := AnsiPos(' ', ss);
   if j > 0 then
    ss := system.Copy(ss, 1, j - 1);
   Result := StrToIntDef(ss, 0);
  end;

begin
  if not FConnection.Connected then
   Connect;
  if (SelectedResource<>nil)and(SelectedResource.IsCollection) then
   FCHeader.Add(Format(davStrDepth,['infinity']));
  if AContent.Size>0 then
   FCHeader.Add(Format(davStrCType,[FContentType]));
  FCHeader.Add(Format(davStrCLength,[AContent.Size]));
  FConnection.Write(FCHeader.Text);
  FConnection.Write(#13#10);
  if AContent.Size>0 then
   FConnection.WriteStream(AContent);
  //Get response
  Repeat
   ReadResponseCode;
   ReadHeaders;
  until FResponseCode<>100;

  if FResponseCode>0 then //-1 response header present
  begin
   if FChunked then
    begin
     FContentLen := BytesForRead;
     while FContentLen > 0 do
      begin
       FConnection.ReadStream(AResponse, FContentLen);
       FConnection.ReadLn;
       FContentLen := BytesForRead;
      end;
      FConnection.ReadLn;
    end
   else
    begin
     if FContentLen<>0 then
      FConnection.ReadStream(AResponse, FContentLen, False);
    end;
  end;
   AResponse.Position:= 0;
end;

function TIEDWebDav.Disconnect: boolean;
begin
 FConnection.Disconnect;
 result:= not FConnection.Connected;
end;

function TIEDWebDav.MkDir(NewName: string): boolean;
begin
  inherited;
  CreateRequest(integer(davMKCOL),FCurrentDir+NewName+'/');
  SendRequest(FContent,FResponse);
  result:=(FResponseCode=201);
end;

function TIEDWebDav.Copy(NewName: string): boolean;
begin
 result:=true;
end;

function TIEDWebDav.Move(NewName: string): boolean;
begin
 result:=true;
end;

function TIEDWebDav.Delete: boolean;
begin
  inherited;
  CheckIsSelected;
  CreateRequest(integer(davDELETE),SelectedResource.ResourceName);
  SendRequest(FContent,FResponse);
  result:=(FResponseCode=201);
end;

function TIEDWebDav.Download(LocalName: string): boolean;
var FS : TFileStream;
begin
  inherited;
  CheckIsSelected;
  CreateRequest(integer(davGET),SelectedResource.ResourceName);
  FS := TFileStream.Create(LocalName,fmCreate);
  try
   SendRequest(FContent,FS);
  finally
   FS.Free;
  end;
  result:=(FResponseCode=200);
end;

function TIEDWebDav.Upload(LocalName: string): boolean;
var FS : TFileStream;
begin
  inherited;
  CheckIsSelected;
  CreateRequest(integer(davPUT),CurrentDir+UrlEncode(ExtractFileName(LocalName)));
  FS := TFileStream.Create(LocalName,fmOpenRead+fmShareDenyWrite);
  try
    SendRequest(FS,FResponse);
  finally
   FS.Free;
  end;
  result:=(FResponseCode=200);
end;

function TIEDWebDav.Lock;
var Content : string;
begin
  inherited;
  CheckIsSelected;
  CreateRequest(integer(davLOCK),SelectedResource.ResourceName);
  FCHeader.Add(Format(davStrTimeout,[4100000000]));
  Content := Format(davContentLock,[DAVLockScopeStr[LockScope],
                                    DAVLockTypeStr[LockType],LockOwner]);

  FContent.WriteBuffer(PChar(Content)^, Length(Content));
  FContentType := 'text/xml; charset="utf-8"';
  SendRequest(FContent,FResponse);
  result:=(FResponseCode=200);
end;

function TIEDWebDav.Unlock: boolean;
begin
  inherited;
  CheckIsSelected;
  CreateRequest(integer(davUNLOCK),SelectedResource.ResourceName);
  FCHeader.Add(Format( davStrLockToken,[SelectedResource.LockEntry.lockToken]));
  SendRequest(FContent,FResponse);
  result:=(FResponseCode=204);
end;

procedure  TIEDWebDav.LoadResourceList;
var MNode, RNode : TIEDXMLNode;
    NewData : TResource;
    i : integer;
begin
  inherited;
  FXML.Root.Clear;
  CreateRequest(integer(davPROPFIND),FCurrentDir);
  FCHeader.Add(Format(davStrDepth,['1']));
  FContent.WriteBuffer(davContentAllProperty, Length(davContentAllProperty));
  FContentType := 'text/xml; charset="utf-8"';
  SendRequest(FContent,FResponse);
  FXML.Parse(FResponse);
  if (FResponseCode<>207) then exit;
  MNode := FXML.Root.NodeOf('MULTISTATUS',[schPartial,schUpper],10);
  if MNode<>nil then
  for i:=0 to MNode.Nodes.Count-1 do
    if MNode.Nodes[i].IsSameName(':RESPONSE') then
     begin
       RNode:= MNode.Nodes[i].NodeOf(':HREF',[schPartial,schUpper],10);
       if RNode=nil then continue;
       NewData := AddNewResource(RNode.Value);
       NewData.ParseXMLnode(MNode.Nodes[i]);
       NewData.CheckData(FHost,FCurrentDir);
     end;
end;

end.
